/**
 * Copyright (C) 2009 fgrilli <federico.grilli@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.google.code.mgnlgroovy.scheduler.admin;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.gui.controlx.list.DefaultValueProvider;
import info.magnolia.cms.gui.controlx.list.ValueProvider;
import info.magnolia.cms.util.NodeDataUtil;

import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateFormatUtils;


/**
 * @author fgrilli
 * @version $Id: JobValueProvider.java 3 2009-04-16 22:48:19Z federico.grilli $
 */
public final class JobValueProvider implements ValueProvider
{

    private final ValueProvider provider = DefaultValueProvider.getInstance();

    /**
     * @param provider
     */
    public JobValueProvider()
    {
    }

    public Object getValue(String name, Object obj)
    {
        Content job = (Content) obj;

        if (name.equals("title"))
        {
            return StringUtils.isNotBlank(job.getName()) ? job.getName() : "no name given";
        }
        else if (name.toLowerCase().endsWith("time"))
        {
            Long timeInMillis = NodeDataUtil.getLong(job, name, 0);
            if (timeInMillis == 0)
                return ""; 
            Date date = new Date(timeInMillis);
            return DateFormatUtils.format(date, "dd MMM yyyy HH:mm:ss");
        }
        else
        {
            // delegate to DefaultValueProvider
            return provider.getValue(name, obj);
        }
    }
}
