/**
 * Copyright (C) 2009 fgrilli <federico.grilli@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.google.code.mgnlgroovy.scheduler.manager;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.util.FactoryUtil;

import java.util.Collection;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.RepositoryException;

import com.google.code.mgnlgroovy.scheduler.JobDefinition;


/**
 * @author fgrilli
 * @version $Id: JobDefinitionManager.java 3 2009-04-16 22:48:19Z federico.grilli $
 */
public interface JobDefinitionManager
{

    Collection<Content> findAllJobDefinitions() throws RepositoryException;

    /**
     * @param definition - an instance of {@link JobDefinition}
     * @return String - the UUID of the newly created or updated {@link JobDefinition}
     * @throws RepositoryException
     * @throws ItemExistsException
     */
    String saveOrUpdateJobDefinition(JobDefinition definition) throws RepositoryException, ItemExistsException;

    void removeJobDefinition(String uuid) throws RepositoryException, ItemNotFoundException;
    
    JobDefinition getJobDefinitionByName(String jobName) throws RepositoryException, ItemNotFoundException;
    
    JobDefinition getJobDefinitionByUUID(String uuid) throws RepositoryException, ItemNotFoundException;
    
    boolean sameJobNameExists(String jobName);

    public class Factory
    {

        public static JobDefinitionManager getInstance()
        {
            return (JobDefinitionManager) FactoryUtil.getSingleton(DefaultJobDefinitionManager.class);
        }
    }
}
